@@ -14,11 +14,7 @@
 #define LISTHANDLER_H_CQ
 
 #include <QtGlobal>
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QtWidgets>
-#else
-#include <QtGui>
-#endif
 
 #include "sqlquery.h"
 
