@@ -6,8 +6,9 @@
 // Author: Jason Haslam
 //
 // Additions Copyright (c) 2011 Archaeopteryx Software, Inc. d/b/a Wingware
-// ScintillaQt.cpp - Qt specific subclass of ScintillaBase
+// @file ScintillaQt.cpp - Qt specific subclass of ScintillaBase
 
+#include <cstdint>
 #include "ScintillaQt.h"
 #include "PlatQt.h"
 
@@ -23,27 +24,26 @@
 #include <QTimer>
 
 using namespace Scintilla;
-
+using namespace Scintilla::Internal;
 
 ScintillaQt::ScintillaQt(QAbstractScrollArea *parent)
 : QObject(parent), scrollArea(parent), vMax(0),  hMax(0), vPage(0), hPage(0),
- haveMouseCapture(false), dragWasDropped(false)
+ haveMouseCapture(false), dragWasDropped(false),
+ rectangularSelectionModifier(SCMOD_ALT)
 {
 
 	wMain = scrollArea->viewport();
 
-	imeInteraction = imeInline;
+	imeInteraction = IMEInteraction::Inline;
 
-	// On OS X drawing text into a pixmap moves it around 1 pixel to
+	// On macOS drawing text into a pixmap moves it around 1 pixel to
 	// the right compared to drawing it directly onto a window.
 	// Buffered drawing turned off by default to avoid this.
 	view.bufferedDraw = false;
 
 	Init();
 
-	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
-		timers[tr] = 0;
-	}
+	std::fill(timers, std::end(timers), 0);
 }
 
 ScintillaQt::~ScintillaQt()
@@ -54,8 +54,8 @@ ScintillaQt::~ScintillaQt()
 
 void ScintillaQt::execCommand(QAction *action)
 {
-	int command = action->data().toInt();
-	Command(command);
+	const int commandNum = action->data().toInt();
+	Command(commandNum);
 }
 
 #if defined(Q_OS_WIN)
@@ -173,25 +173,24 @@ static QString StringFromSelectedText(const SelectionText &selectedText)
 	}
 }
 
-static void AddRectangularToMime(QMimeData *mimeData, QString su)
+static void AddRectangularToMime(QMimeData *mimeData, [[maybe_unused]] const QString &su)
 {
 #if defined(Q_OS_WIN)
 	// Add an empty marker
 	mimeData->setData(sMSDEVColumnSelect, QByteArray());
 #elif defined(Q_OS_MAC)
-	// OS X gets marker + data to work with other implementations.
+	// macOS gets marker + data to work with other implementations.
 	// Don't understand how this works but it does - the
 	// clipboard format is supposed to be UTF-16, not UTF-8.
 	mimeData->setData(sScintillaRecMimeType, su.toUtf8());
 #else
-	Q_UNUSED(su);
 	// Linux
 	// Add an empty marker
 	mimeData->setData(sMimeRectangularMarker, QByteArray());
 #endif
 }
 
-static void AddLineCutCopyToMime(QMimeData *mimeData)
+static void AddLineCutCopyToMime([[maybe_unused]] QMimeData *mimeData)
 {
 #if defined(Q_OS_WIN)
 	// Add an empty marker
@@ -251,6 +250,28 @@ bool ScintillaQt::ValidCodePage(int codePage) const
 	|| codePage == 1361;
 }
 
+std::string ScintillaQt::UTF8FromEncoded(std::string_view encoded) const {
+	if (IsUnicodeMode()) {
+		return std::string(encoded);
+	} else {
+		QTextCodec *codec = QTextCodec::codecForName(
+				CharacterSetID(CharacterSetOfDocument()));
+		QString text = codec->toUnicode(encoded.data(), static_cast<int>(encoded.length()));
+		return text.toStdString();
+	}
+}
+
+std::string ScintillaQt::EncodedFromUTF8(std::string_view utf8) const {
+	if (IsUnicodeMode()) {
+		return std::string(utf8);
+	} else {
+		QString text = QString::fromUtf8(utf8.data(), static_cast<int>(utf8.length()));
+		QTextCodec *codec = QTextCodec::codecForName(
+				CharacterSetID(CharacterSetOfDocument()));
+		QByteArray ba = codec->fromUnicode(text);
+		return std::string(ba.data(), ba.length());
+	}
+}
 
 void ScintillaQt::ScrollText(Sci::Line linesToMove)
 {
@@ -366,9 +387,9 @@ void ScintillaQt::PasteFromMode(QClipboard::Mode clipboardMode_)
 	selText.Copy(dest, pdoc->dbcsCodePage, CharacterSetOfDocument(), isRectangular, false);
 
 	UndoGroup ug(pdoc);
-	ClearSelection(multiPasteMode == SC_MULTIPASTE_EACH);
+	ClearSelection(multiPasteMode == MultiPaste::Each);
 	InsertPasteShape(selText.Data(), selText.Length(),
-		isRectangular ? pasteRectangular : (isLine ? pasteLine : pasteStream));
+		isRectangular ? PasteShape::rectangular : (isLine ? PasteShape::line : PasteShape::stream));
 	EnsureCaretVisible();
 }
 
@@ -413,7 +434,7 @@ void ScintillaQt::NotifyFocus(bool focus)
 	Editor::NotifyFocus(focus);
 }
 
-void ScintillaQt::NotifyParent(SCNotification scn)
+void ScintillaQt::NotifyParent(NotificationData scn)
 {
 	scn.nmhdr.hwndFrom = wMain.GetID();
 	scn.nmhdr.idFrom = GetCtrlID();
@@ -422,8 +443,8 @@ void ScintillaQt::NotifyParent(SCNotification scn)
 
 void ScintillaQt::NotifyURIDropped(const char *uri)
 {
-	SCNotification scn = {};
-	scn.nmhdr.code = SCN_URIDROPPED;
+	NotificationData scn = {};
+	scn.nmhdr.code = Notification::URIDropped;
 	scn.text = uri;
 
 	NotifyParent(scn);
@@ -431,20 +452,20 @@ void ScintillaQt::NotifyURIDropped(const char *uri)
 
 bool ScintillaQt::FineTickerRunning(TickReason reason)
 {
-	return timers[reason] != 0;
+	return timers[static_cast<size_t>(reason)] != 0;
 }
 
 void ScintillaQt::FineTickerStart(TickReason reason, int millis, int /* tolerance */)
 {
 	FineTickerCancel(reason);
-	timers[reason] = startTimer(millis);
+	timers[static_cast<size_t>(reason)] = startTimer(millis);
 }
 
 // CancelTimers cleans up all fine-ticker timers and is non-virtual to avoid warnings when
 // called during destruction.
 void ScintillaQt::CancelTimers()
 {
-	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
+	for (size_t tr = static_cast<size_t>(TickReason::caret); tr <= static_cast<size_t>(TickReason::dwell); tr++) {
 		if (timers[tr]) {
 			killTimer(timers[tr]);
 			timers[tr] = 0;
@@ -454,9 +475,10 @@ void ScintillaQt::CancelTimers()
 
 void ScintillaQt::FineTickerCancel(TickReason reason)
 {
-	if (timers[reason]) {
-		killTimer(timers[reason]);
-		timers[reason] = 0;
+	const size_t reasonIndex = static_cast<size_t>(reason);
+	if (timers[reasonIndex]) {
+		killTimer(timers[reasonIndex]);
+		timers[reasonIndex] = 0;
 	}
 }
 
@@ -470,12 +492,11 @@ void ScintillaQt::onIdle()
 
 bool ScintillaQt::ChangeIdle(bool on)
 {
-	QTimer *qIdle;
 	if (on) {
 		// Start idler, if it's not running.
 		if (!idler.state) {
 			idler.state = true;
-			qIdle = new QTimer;
+			QTimer *qIdle = new QTimer;
 			connect(qIdle, SIGNAL(timeout()), this, SLOT(onIdle()));
 			qIdle->start(0);
 			idler.idlerID = qIdle;
@@ -484,7 +505,7 @@ bool ScintillaQt::ChangeIdle(bool on)
 		// Stop idler, if it's running
 		if (idler.state) {
 			idler.state = false;
-			qIdle = static_cast<QTimer *>(idler.idlerID);
+			QTimer *qIdle = static_cast<QTimer *>(idler.idlerID);
 			qIdle->stop();
 			disconnect(qIdle, SIGNAL(timeout()), nullptr, nullptr);
 			delete qIdle;
@@ -499,7 +520,7 @@ bool ScintillaQt::SetIdle(bool on)
 	return ChangeIdle(on);
 }
 
-int ScintillaQt::CharacterSetOfDocument() const
+CharacterSet ScintillaQt::CharacterSetOfDocument() const
 {
 	return vs.styles[STYLE_DEFAULT].characterSet;
 }
@@ -531,12 +552,12 @@ QByteArray ScintillaQt::BytesForDocument(const QString &text) const
 	}
 }
 
+namespace {
 
 class CaseFolderDBCS : public CaseFolderTable {
 	QTextCodec *codec;
 public:
 	explicit CaseFolderDBCS(QTextCodec *codec_) : codec(codec_) {
-		StandardASCII();
 	}
 	size_t Fold(char *folded, size_t sizeFolded, const char *mixed, size_t lenMixed) override {
 		if ((lenMixed == 1) && (sizeFolded > 0)) {
@@ -558,16 +579,17 @@ public:
 	}
 };
 
-CaseFolder *ScintillaQt::CaseFolderForEncoding()
+}
+
+std::unique_ptr<CaseFolder> ScintillaQt::CaseFolderForEncoding()
 {
 	if (pdoc->dbcsCodePage == SC_CP_UTF8) {
-		return new CaseFolderUnicode();
+		return std::make_unique<CaseFolderUnicode>();
 	} else {
 		const char *charSetBuffer = CharacterSetIDOfDocument();
 		if (charSetBuffer) {
 			if (pdoc->dbcsCodePage == 0) {
-				CaseFolderTable *pcf = new CaseFolderTable();
-				pcf->StandardASCII();
+				std::unique_ptr<CaseFolderTable> pcf = std::make_unique<CaseFolderTable>();
 				QTextCodec *codec = QTextCodec::codecForName(charSetBuffer);
 				// Only for single byte encodings
 				for (int i=0x80; i<0x100; i++) {
@@ -584,22 +606,22 @@ CaseFolder *ScintillaQt::CaseFolderForEncoding()
 				}
 				return pcf;
 			} else {
-				return new CaseFolderDBCS(QTextCodec::codecForName(charSetBuffer));
+				return std::make_unique<CaseFolderDBCS>(QTextCodec::codecForName(charSetBuffer));
 			}
 		}
 		return nullptr;
 	}
 }
 
-std::string ScintillaQt::CaseMapString(const std::string &s, int caseMapping)
+std::string ScintillaQt::CaseMapString(const std::string &s, CaseMapping caseMapping)
 {
-	if ((s.size() == 0) || (caseMapping == cmSame))
+	if (s.empty() || (caseMapping == CaseMapping::same))
 		return s;
 
 	if (IsUnicodeMode()) {
 		std::string retMapped(s.length() * maxExpansionCaseConversion, 0);
 		size_t lenMapped = CaseConvertString(&retMapped[0], retMapped.length(), s.c_str(), s.length(),
-			(caseMapping == cmUpper) ? CaseConversionUpper : CaseConversionLower);
+			(caseMapping == CaseMapping::upper) ? CaseConversion::upper : CaseConversion::lower);
 		retMapped.resize(lenMapped);
 		return retMapped;
 	}
@@ -607,7 +629,7 @@ std::string ScintillaQt::CaseMapString(const std::string &s, int caseMapping)
 	QTextCodec *codec = QTextCodec::codecForName(CharacterSetIDOfDocument());
 	QString text = codec->toUnicode(s.c_str(), static_cast<int>(s.length()));
 
-	if (caseMapping == cmUpper) {
+	if (caseMapping == CaseMapping::upper) {
 		text = text.toUpper();
 	} else {
 		text = text.toLower();
@@ -632,7 +654,7 @@ bool ScintillaQt::HaveMouseCapture()
 
 void ScintillaQt::StartDrag()
 {
-	inDragDrop = ddDragging;
+	inDragDrop = DragDrop::dragging;
 	dropWentOutside = true;
 	if (drag.Length()) {
 		QMimeData *mimeData = new QMimeData;
@@ -645,19 +667,19 @@ void ScintillaQt::StartDrag()
 		QDrag *dragon = new QDrag(scrollArea);
 		dragon->setMimeData(mimeData);
 
-		Qt::DropAction dropAction = dragon->exec(Qt::CopyAction|Qt::MoveAction);
+		Qt::DropAction dropAction = dragon->exec(static_cast<Qt::DropActions>(Qt::CopyAction|Qt::MoveAction));
 		if ((dropAction == Qt::MoveAction) && dropWentOutside) {
 			// Remove dragged out text
 			ClearSelection();
 		}
 	}
-	inDragDrop = ddNone;
+	inDragDrop = DragDrop::none;
 	SetDragPosition(SelectionPosition(Sci::invalidPosition));
 }
 
 class CallTipImpl : public QWidget {
 public:
-	CallTipImpl(CallTip *pct_)
+	explicit CallTipImpl(CallTip *pct_)
 		: QWidget(nullptr, Qt::ToolTip),
 		  pct(pct_)
 	{
@@ -669,12 +691,10 @@ public:
 	void paintEvent(QPaintEvent *) override
 	{
 		if (pct->inCallTipMode) {
-			Surface *surfaceWindow = Surface::Allocate(0);
+			std::unique_ptr<Surface> surfaceWindow = Surface::Allocate(Technology::Default);
 			surfaceWindow->Init(this);
-			surfaceWindow->SetUnicodeMode(SC_CP_UTF8 == pct->codePage);
-			surfaceWindow->SetDBCSMode(pct->codePage);
-			pct->PaintCT(surfaceWindow);
-			delete surfaceWindow;
+			surfaceWindow->SetMode(SurfaceMode(pct->codePage, false));
+			pct->PaintCT(surfaceWindow.get());
 		}
 	}
 
@@ -710,41 +730,51 @@ void ScintillaQt::AddToPopUp(const char *label,
 
 	// Make sure the menu's signal is connected only once.
 	menu->disconnect();
-	connect(menu, SIGNAL(triggered(QAction *)),
-	        this, SLOT(execCommand(QAction *)));
+	connect(menu, SIGNAL(triggered(QAction*)),
+		this, SLOT(execCommand(QAction*)));
 }
 
-sptr_t ScintillaQt::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam)
+sptr_t ScintillaQt::WndProc(Message iMessage, uptr_t wParam, sptr_t lParam)
 {
 	try {
 		switch (iMessage) {
 
-		case SCI_SETIMEINTERACTION:
+		case Message::SetIMEInteraction:
 			// Only inline IME supported on Qt
 			break;
 
-		case SCI_GRABFOCUS:
+		case Message::GrabFocus:
 			scrollArea->setFocus(Qt::OtherFocusReason);
 			break;
 
-		case SCI_GETDIRECTFUNCTION:
+		case Message::GetDirectFunction:
 			return reinterpret_cast<sptr_t>(DirectFunction);
 
-		case SCI_GETDIRECTPOINTER:
+		case Message::GetDirectStatusFunction:
+			return reinterpret_cast<sptr_t>(DirectStatusFunction);
+
+		case Message::GetDirectPointer:
 			return reinterpret_cast<sptr_t>(this);
 
+		case Message::SetRectangularSelectionModifier:
+			rectangularSelectionModifier = static_cast<int>(wParam);
+			break;
+
+		case Message::GetRectangularSelectionModifier:
+			return rectangularSelectionModifier;
+
 		default:
 			return ScintillaBase::WndProc(iMessage, wParam, lParam);
 		}
 	} catch (std::bad_alloc &) {
-		errorStatus = SC_STATUS_BADALLOC;
+		errorStatus = Status::BadAlloc;
 	} catch (...) {
-		errorStatus = SC_STATUS_FAILURE;
+		errorStatus = Status::Failure;
 	}
 	return 0;
 }
 
-sptr_t ScintillaQt::DefWndProc(unsigned int, uptr_t, sptr_t)
+sptr_t ScintillaQt::DefWndProc(Message, uptr_t, sptr_t)
 {
 	return 0;
 }
@@ -752,7 +782,17 @@ sptr_t ScintillaQt::DefWndProc(unsigned int, uptr_t, sptr_t)
 sptr_t ScintillaQt::DirectFunction(
     sptr_t ptr, unsigned int iMessage, uptr_t wParam, sptr_t lParam)
 {
-	return reinterpret_cast<ScintillaQt *>(ptr)->WndProc(iMessage, wParam, lParam);
+	ScintillaQt *sci = reinterpret_cast<ScintillaQt *>(ptr);
+	return sci->WndProc(static_cast<Message>(iMessage), wParam, lParam);
+}
+
+sptr_t ScintillaQt::DirectStatusFunction(
+    sptr_t ptr, unsigned int iMessage, uptr_t wParam, sptr_t lParam, int *pStatus)
+{
+	ScintillaQt *sci = reinterpret_cast<ScintillaQt *>(ptr);
+	const sptr_t returnValue = sci->WndProc(static_cast<Message>(iMessage), wParam, lParam);
+	*pStatus = static_cast<int>(sci->errorStatus);
+	return returnValue;
 }
 
 // Additions to merge in Scientific Toolworks widget structure
@@ -760,7 +800,7 @@ sptr_t ScintillaQt::DirectFunction(
 void ScintillaQt::PartialPaint(const PRectangle &rect)
 {
 	rcPaint = rect;
-	paintState = painting;
+	paintState = PaintState::painting;
 	PRectangle rcClient = GetClientRectangle();
 	paintingAllText = rcPaint.Contains(rcClient);
 
@@ -768,11 +808,11 @@ void ScintillaQt::PartialPaint(const PRectangle &rect)
 	Paint(surfacePaint, rcPaint);
 	surfacePaint->Release();
 
-	if (paintState == paintAbandoned) {
+	if (paintState == PaintState::abandoned) {
 		// FIXME: Failure to paint the requested rectangle in each
 		// paint event causes flicker on some platforms (Mac?)
 		// Paint rect immediately.
-		paintState = painting;
+		paintState = PaintState::painting;
 		paintingAllText = true;
 
 		AutoSurface surface(this);
@@ -783,7 +823,7 @@ void ScintillaQt::PartialPaint(const PRectangle &rect)
 		scrollArea->viewport()->update();
 	}
 
-	paintState = notPainting;
+	paintState = PaintState::notPainting;
 }
 
 void ScintillaQt::DragEnter(const Point &point)
@@ -825,9 +865,9 @@ void ScintillaQt::DropUrls(const QMimeData *data)
 
 void ScintillaQt::timerEvent(QTimerEvent *event)
 {
-	for (TickReason tr=tickCaret; tr<=tickDwell; tr = static_cast<TickReason>(tr+1)) {
+	for (size_t tr=static_cast<size_t>(TickReason::caret); tr<=static_cast<size_t>(TickReason::dwell); tr++) {
 		if (timers[tr] == event->timerId()) {
-			TickFor(tr);
+			TickFor(static_cast<TickReason>(tr));
 		}
 	}
 }
