@@ -202,9 +202,11 @@
 
 #include <cstddef>
 #include <cstdlib>
+#include <cstdint>
 
 #include <stdexcept>
 #include <string>
+#include <array>
 #include <algorithm>
 #include <iterator>
 
@@ -212,7 +214,7 @@
 #include "CharClassify.h"
 #include "RESearch.h"
 
-using namespace Scintilla;
+using namespace Scintilla::Internal;
 
 #define OKP     1
 #define NOP     0
@@ -237,11 +239,8 @@ using namespace Scintilla;
  * The following defines are not meant to be changeable.
  * They are for readability only.
  */
-#define BLKIND  0370
 #define BITIND  07
 
-static const char bitarr[] = { 1, 2, 4, 8, 16, 32, 64, '\200' };
-
 #define badpat(x)	(*nfa = END, x)
 
 /*
@@ -255,39 +254,19 @@ RESearch::RESearch(CharClassify *charClassTable) {
 	failure = 0;
 	charClass = charClassTable;
 	sta = NOP;                  /* status of lastpat */
-	bol = 0;
-	constexpr unsigned char nul = 0;
-	std::fill(bittab, std::end(bittab), nul);
-	std::fill(tagstk, std::end(tagstk), 0);
-	std::fill(nfa, std::end(nfa), '\0');
+	lineStartPos = 0;
+	lineEndPos = 0;
+	nfa[0] = END;
 	Clear();
 }
 
-RESearch::~RESearch() {
-	Clear();
-}
-
-void RESearch::Clear() noexcept {
-	for (int i = 0; i < MAXTAG; i++) {
-		pat[i].clear();
-		bopat[i] = NOTFOUND;
-		eopat[i] = NOTFOUND;
-	}
-}
-
-void RESearch::GrabMatches(const CharacterIndexer &ci) {
-	for (unsigned int i = 0; i < MAXTAG; i++) {
-		if ((bopat[i] != NOTFOUND) && (eopat[i] != NOTFOUND)) {
-			const Sci::Position len = eopat[i] - bopat[i];
-			pat[i].resize(len);
-			for (Sci::Position j = 0; j < len; j++)
-				pat[i][j] = ci.CharAt(bopat[i] + j);
-		}
-	}
+void RESearch::Clear() {
+	bopat.fill(NOTFOUND);
+	eopat.fill(NOTFOUND);
 }
 
 void RESearch::ChSet(unsigned char c) noexcept {
-	bittab[((c) & BLKIND) >> 3] |= bitarr[(c) & BITIND];
+	bittab[c >> 3] |= 1 << (c & BITIND);
 }
 
 void RESearch::ChSetWithCase(unsigned char c, bool caseSensitive) noexcept {
@@ -301,7 +280,9 @@ void RESearch::ChSetWithCase(unsigned char c, bool caseSensitive) noexcept {
 	}
 }
 
-static unsigned char escapeValue(unsigned char ch) noexcept {
+namespace {
+
+constexpr unsigned char escapeValue(unsigned char ch) noexcept {
 	switch (ch) {
 	case 'a':	return '\a';
 	case 'b':	return '\b';
@@ -310,11 +291,12 @@ static unsigned char escapeValue(unsigned char ch) noexcept {
 	case 'r':	return '\r';
 	case 't':	return '\t';
 	case 'v':	return '\v';
+	default:	break;
 	}
 	return 0;
 }
 
-static int GetHexaChar(unsigned char hd1, unsigned char hd2) noexcept {
+constexpr int GetHexaChar(unsigned char hd1, unsigned char hd2) noexcept {
 	int hexValue = 0;
 	if (hd1 >= '0' && hd1 <= '9') {
 		hexValue += 16 * (hd1 - '0');
@@ -337,6 +319,12 @@ static int GetHexaChar(unsigned char hd1, unsigned char hd2) noexcept {
 	return hexValue;
 }
 
+constexpr int isinset(const char *ap, unsigned char c) noexcept {
+	return ap[c >> 3] & (1 << (c & BITIND));
+}
+
+}
+
 /**
  * Called when the parser finds a backslash not followed
  * by a valid expression (like \( in non-Posix mode).
@@ -345,14 +333,11 @@ static int GetHexaChar(unsigned char hd1, unsigned char hd2) noexcept {
  * @return the char if it resolves to a simple char,
  * or -1 for a char class. In this case, bittab is changed.
  */
-int RESearch::GetBackslashExpression(
-    const char *pattern,
-    int &incr) noexcept {
+int RESearch::GetBackslashExpression(const char *pattern, int &incr) noexcept {
 	// Since error reporting is primitive and messages are not used anyway,
 	// I choose to interpret unexpected syntax in a logical way instead
 	// of reporting errors. Otherwise, we can stick on, eg., PCRE behaviour.
 	incr = 0;	// Most of the time, will skip the char "naturally".
-	int c;
 	int result = -1;
 	const unsigned char bsc = *pattern;
 	if (!bsc) {
@@ -384,12 +369,12 @@ int RESearch::GetBackslashExpression(
 		}
 		break;
 	case 'd':
-		for (c = '0'; c <= '9'; c++) {
+		for (int c = '0'; c <= '9'; c++) {
 			ChSet(static_cast<unsigned char>(c));
 		}
 		break;
 	case 'D':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (c < '0' || c > '9') {
 				ChSet(static_cast<unsigned char>(c));
 			}
@@ -404,21 +389,21 @@ int RESearch::GetBackslashExpression(
 		ChSet('\v');
 		break;
 	case 'S':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (c != ' ' && !(c >= 0x09 && c <= 0x0D)) {
 				ChSet(static_cast<unsigned char>(c));
 			}
 		}
 		break;
 	case 'w':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (iswordc(static_cast<unsigned char>(c))) {
 				ChSet(static_cast<unsigned char>(c));
 			}
 		}
 		break;
 	case 'W':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (!iswordc(static_cast<unsigned char>(c))) {
 				ChSet(static_cast<unsigned char>(c));
 			}
@@ -430,32 +415,32 @@ int RESearch::GetBackslashExpression(
 	return result;
 }
 
-const char *RESearch::Compile(const char *pattern, Sci::Position length, bool caseSensitive, bool posix) noexcept {
-	char *mp=nfa;          /* nfa pointer       */
-	char *lp;              /* saved pointer     */
-	char *sp=nfa;          /* another one       */
-	char *mpMax = mp + MAXNFA - BITBLK - 10;
-
-	int tagi = 0;          /* tag stack index   */
-	int tagc = 1;          /* actual tag count  */
-
-	int n;
-	char mask;             /* xor mask -CCL/NCL */
-	int c1, c2, prevChar;
-
+const char *RESearch::Compile(const char *pattern, Sci::Position length, bool caseSensitive, bool posix) {
 	if (!pattern || !length) {
 		if (sta)
 			return nullptr;
 		else
 			return badpat("No previous regular expression");
 	}
+
+	bittab.fill(0);
+	nfa[0] = END;
+
+	char *mp=nfa;          /* nfa pointer       */
+	char *sp=nfa;          /* another saved pointer */
+	const char * const mpMax = mp + MAXNFA - BITBLK - 10;
+
+	int tagstk[MAXTAG]{};  /* subpat tag stack */
+	int tagi = 0;          /* tag stack index   */
+	int tagc = 1;          /* actual tag count  */
+
 	sta = NOP;
 
 	const char *p=pattern;     /* pattern pointer   */
 	for (int i=0; i<length; i++, p++) {
 		if (mp > mpMax)
 			return badpat("Pattern too long");
-		lp = mp;
+		char *lp = mp;			/* saved pointer     */
 		switch (*p) {
 
 		case '.':               /* match any char  */
@@ -472,7 +457,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			break;
 
 		case '$':               /* match endofline */
-			if (!*(p+1)) {
+			if (!p[1]) {
 				*mp++ = EOL;
 			} else {
 				*mp++ = CHR;
@@ -480,17 +465,15 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			}
 			break;
 
-		case '[':               /* match char class */
-			*mp++ = CCL;
-			prevChar = 0;
+		case '[': {               /* match char class */
+			int prevChar = 0;
+			char mask = 0;      /* xor mask -CCL/NCL */
 
 			i++;
 			if (*++p == '^') {
 				mask = '\377';
 				i++;
 				p++;
-			} else {
-				mask = 0;
 			}
 
 			if (*p == '-') {	/* real dash */
@@ -509,18 +492,18 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 						// Previous def. was a char class like \d, take dash literally
 						prevChar = *p;
 						ChSet(*p);
-					} else if (*(p+1)) {
-						if (*(p+1) != ']') {
-							c1 = prevChar + 1;
+					} else if (p[1]) {
+						if (p[1] != ']') {
+							int c1 = prevChar + 1;
 							i++;
-							c2 = static_cast<unsigned char>(*++p);
+							int c2 = static_cast<unsigned char>(*++p);
 							if (c2 == '\\') {
-								if (!*(p+1)) {	// End of RE
+								if (!p[1]) {	// End of RE
 									return badpat("Missing ]");
 								} else {
 									i++;
 									p++;
-									int incr;
+									int incr = 0;
 									c2 = GetBackslashExpression(p, incr);
 									i += incr;
 									p += incr;
@@ -552,10 +535,10 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 					} else {
 						return badpat("Missing ]");
 					}
-				} else if (*p == '\\' && *(p+1)) {
+				} else if (*p == '\\' && p[1]) {
 					i++;
 					p++;
-					int incr;
+					int incr = 0;
 					const int c = GetBackslashExpression(p, incr);
 					i += incr;
 					p += incr;
@@ -577,10 +560,12 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			if (!*p)
 				return badpat("Missing ]");
 
-			for (n = 0; n < BITBLK; bittab[n++] = 0)
-				*mp++ = static_cast<char>(mask ^ bittab[n]);
-
-			break;
+			*mp++ = CCL;
+			for (const unsigned char byte : bittab) {
+				*mp++ = mask ^ byte;
+			}
+			bittab.fill(0);
+		} break;
 
 		case '*':               /* match 0 or more... */
 		case '+':               /* match 1 or more... */
@@ -614,7 +599,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			while (--mp > lp)
 				*mp = mp[-1];
 			if (*p == '?')          *mp = CLQ;
-			else if (*(p+1) == '?') *mp = LCLO;
+			else if (p[1] == '?') *mp = LCLO;
 			else                    *mp = CLO;
 
 			mp = sp;
@@ -639,8 +624,8 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			case '6':
 			case '7':
 			case '8':
-			case '9':
-				n = *p-'0';
+			case '9': {
+				const int n = *p-'0';
 				if (tagi > 0 && tagstk[tagi] == n)
 					return badpat("Cyclical reference");
 				if (tagc > n) {
@@ -649,7 +634,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 				} else {
 					return badpat("Undetermined reference");
 				}
-				break;
+			} break;
 			default:
 				if (!posix && *p == '(') {
 					if (tagc < MAXTAG) {
@@ -669,8 +654,8 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 						return badpat("Unmatched \\)");
 					}
 				} else {
-					int incr;
-					int c = GetBackslashExpression(p, incr);
+					int incr = 0;
+					const int c = GetBackslashExpression(p, incr);
 					i += incr;
 					p += incr;
 					if (c >= 0) {
@@ -678,9 +663,10 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 						*mp++ = static_cast<unsigned char>(c);
 					} else {
 						*mp++ = CCL;
-						mask = 0;
-						for (n = 0; n < BITBLK; bittab[n++] = 0)
-							*mp++ = static_cast<char>(mask ^ bittab[n]);
+						for (const unsigned char byte : bittab) {
+							*mp++ = byte;
+						}
+						bittab.fill(0);
 					}
 				}
 			}
@@ -712,11 +698,12 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 					*mp++ = CHR;
 					*mp++ = c;
 				} else {
-					*mp++ = CCL;
-					mask = 0;
 					ChSetWithCase(c, false);
-					for (n = 0; n < BITBLK; bittab[n++] = 0)
-						*mp++ = static_cast<char>(mask ^ bittab[n]);
+					*mp++ = CCL;
+					for (const unsigned char byte : bittab) {
+						*mp++ = byte;
+					}
+					bittab.fill(0);
 				}
 			}
 			break;
@@ -752,11 +739,9 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
  *
  */
 int RESearch::Execute(const CharacterIndexer &ci, Sci::Position lp, Sci::Position endp) {
-	unsigned char c;
 	Sci::Position ep = NOTFOUND;
-	char *ap = nfa;
+	const char * const ap = nfa;
 
-	bol = lp;
 	failure = 0;
 
 	Clear();
@@ -767,34 +752,54 @@ int RESearch::Execute(const CharacterIndexer &ci, Sci::Position lp, Sci::Positio
 		ep = PMatch(ci, lp, endp, ap);
 		break;
 	case EOL:			/* just searching for end of line normal path doesn't work */
-		if (*(ap+1) == END) {
+		if (endp == lineEndPos && ap[1] == END) {
 			lp = endp;
 			ep = lp;
 			break;
 		} else {
 			return 0;
 		}
-	case CHR:			/* ordinary char: locate it fast */
-		c = *(ap+1);
+	case CHR: {			/* ordinary char: locate it fast */
+		const unsigned char c = ap[1];
 		while ((lp < endp) && (static_cast<unsigned char>(ci.CharAt(lp)) != c))
 			lp++;
 		if (lp >= endp)	/* if EOS, fail, else fall through. */
 			return 0;
-		// Falls through.
+	}
+	[[fallthrough]];
 	default:			/* regular matching all the way. */
 		while (lp < endp) {
 			ep = PMatch(ci, lp, endp, ap);
-			if (ep != NOTFOUND)
-				break;
+			if (ep != NOTFOUND) {
+				// fix match started from middle of character like DBCS trailing ASCII byte
+				const Sci::Position pos = ci.MovePositionOutsideChar(lp, -1);
+				if (pos != lp) {
+					ep = NOTFOUND;
+				} else {
+					break;
+				}
+			}
 			lp++;
 		}
 		break;
 	case END:			/* munged automaton. fail always */
 		return 0;
 	}
-	if (ep == NOTFOUND)
-		return 0;
+	if (ep == NOTFOUND) {
+		/* similar to EOL, match EOW at line end */
+		if (endp == lineEndPos && *ap == EOW) {
+			if ((ap[1] == END || ((ap[1] == EOL && ap[2] == END))) && iswordc(ci.CharAt(lp - 1))) {
+				lp = endp;
+				ep = lp;
+			} else {
+				return 0;
+			}
+		} else {
+			return 0;
+		}
+	}
 
+	ep = ci.MovePositionOutsideChar(ep, 1);
 	bopat[0] = lp;
 	eopat[0] = ep;
 	return 1;
@@ -829,11 +834,7 @@ int RESearch::Execute(const CharacterIndexer &ci, Sci::Position lp, Sci::Positio
  *  by tagged expressions (n = 1 to 9).
  */
 
-extern void re_fail(char *,char);
-
-static inline int isinset(const char *ap, unsigned char c) noexcept {
-	return ap[(c & BLKIND) >> 3] & bitarr[c & BITIND];
-}
+//extern void re_fail(char *,char);
 
 /*
  * skip values for CLO XXX to skip past the closure
@@ -843,13 +844,8 @@ static inline int isinset(const char *ap, unsigned char c) noexcept {
 #define CHRSKIP 3	/* [CLO] CHR chr END      */
 #define CCLSKIP 34	/* [CLO] CCL 32 bytes END */
 
-Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci::Position endp, char *ap) {
-	int op, c, n;
-	Sci::Position e;		/* extra pointer for CLO  */
-	Sci::Position bp;		/* beginning of subpat... */
-	Sci::Position ep;		/* ending of subpat...    */
-	Sci::Position are;	/* to save the line ptr.  */
-	Sci::Position llp;	/* lazy lp for LCLO       */
+Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci::Position endp, const char *ap) {
+	unsigned char op = 0;
 
 	while ((op = *ap++) != END)
 		switch (op) {
@@ -870,39 +866,44 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 			ap += BITBLK;
 			break;
 		case BOL:
-			if (lp != bol)
+			if (lp != lineStartPos)
 				return NOTFOUND;
 			break;
 		case EOL:
-			if (lp < endp)
+			if (lp < lineEndPos)
 				return NOTFOUND;
 			break;
 		case BOT:
-			bopat[static_cast<int>(*ap++)] = lp;
+			if (lp != ci.MovePositionOutsideChar(lp, -1)) {
+				return NOTFOUND;
+			}
+			bopat[static_cast<unsigned char>(*ap++)] = lp;
 			break;
 		case EOT:
-			eopat[static_cast<int>(*ap++)] = lp;
+			lp = ci.MovePositionOutsideChar(lp, 1);
+			eopat[static_cast<unsigned char>(*ap++)] = lp;
 			break;
 		case BOW:
-			if ((lp!=bol && iswordc(ci.CharAt(lp-1))) || !iswordc(ci.CharAt(lp)))
+			if ((lp!=lineStartPos && iswordc(ci.CharAt(lp-1))) || !iswordc(ci.CharAt(lp)))
 				return NOTFOUND;
 			break;
 		case EOW:
-			if (lp==bol || !iswordc(ci.CharAt(lp-1)) || iswordc(ci.CharAt(lp)))
+			if (lp==lineStartPos || !iswordc(ci.CharAt(lp-1)) || iswordc(ci.CharAt(lp)))
 				return NOTFOUND;
 			break;
-		case REF:
-			n = *ap++;
-			bp = bopat[n];
-			ep = eopat[n];
+		case REF: {
+			const int n = static_cast<unsigned char>(*ap++);
+			Sci::Position bp = bopat[n];		/* beginning of subpat... */
+			const Sci::Position ep = eopat[n];	/* ending of subpat...    */
 			while (bp < ep)
 				if (ci.CharAt(bp++) != ci.CharAt(lp++))
 					return NOTFOUND;
-			break;
+		} break;
 		case LCLO:
 		case CLQ:
-		case CLO:
-			are = lp;
+		case CLO: {
+			int n = 0;
+			const Sci::Position are = lp;	/* to save the line ptr.  */
 			switch (*ap) {
 
 			case ANY:
@@ -914,15 +915,15 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 
 				n = ANYSKIP;
 				break;
-			case CHR:
-				c = *(ap+1);
+			case CHR: {
+				const char c = ap[1];
 				if (op == CLO || op == LCLO)
 					while ((lp < endp) && (c == ci.CharAt(lp)))
 						lp++;
 				else if ((lp < endp) && (c == ci.CharAt(lp)))
 					lp++;
 				n = CHRSKIP;
-				break;
+			} break;
 			case CCL:
 				while ((lp < endp) && isinset(ap+1, ci.CharAt(lp)))
 					lp++;
@@ -935,11 +936,11 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 			}
 			ap += n;
 
-			llp = lp;
-			e = NOTFOUND;
+			Sci::Position llp = lp;		/* lazy lp for LCLO       */
+			Sci::Position e = NOTFOUND; /* extra pointer for CLO  */
 			while (llp >= are) {
-				Sci::Position q;
-				if ((q = PMatch(ci, llp, endp, ap)) != NOTFOUND) {
+				const Sci::Position q = PMatch(ci, llp, endp, ap);
+				if (q != NOTFOUND) {
 					e = q;
 					lp = llp;
 					if (op != LCLO) return e;
@@ -950,11 +951,10 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 			if (*ap == EOT)
 				PMatch(ci, lp, endp, ap);
 			return e;
+		}
 		default:
 			//re_fail("RESearch::Execute: bad nfa.", static_cast<char>(op));
 			return NOTFOUND;
 		}
 	return lp;
 }
-
-
